/**
# Copyright 2009-2010 The Padre development team as listed in Padre.pm.
# LICENSE
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl 5 itself.
*/
/**
 * @author Olivier Mengué <dolmen@cpan.org>
 * @author Ahmad M. Zawawi <ahmad.zawawi@gmail.com>
 */

#include <windef.h>
#include <winuser.h>
#include <winver.h>

#define IDI_APP           101

#define IDS_APP_TITLE     110
#define IDS_ERR_WPERL     111
#define IDS_ERR_SCRIPT    112
#define IDS_ERR_RUN       113
