package Padre::Wx::FBP::Sync;

# This module was generated by Padre::Plugin::FormBuilder::Perl.
# To change this module, edit the original .fbp file and regenerate.
# DO NOT MODIFY BY HAND!

use 5.008;
use strict;
use warnings;
use Padre::Wx ();
use Padre::Wx::Role::Main ();

our $VERSION = '0.86';
our @ISA     = qw{
	Padre::Wx::Role::Main
	Wx::Dialog
};

sub new {
	my $class  = shift;
	my $parent = shift;

	my $self = $class->SUPER::new(
		$parent,
		-1,
		Wx::gettext("Padre Sync"),
		Wx::wxDefaultPosition,
		Wx::wxDefaultSize,
		Wx::wxDEFAULT_DIALOG_STYLE,
	);

	my $m_staticText12 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Server"),
	);

	my $txt_remote = Wx::TextCtrl->new(
		$self,
		-1,
		"http://sync.perlide.org/",
		Wx::wxDefaultPosition,
		Wx::wxDefaultSize,
	);

	my $m_staticText13 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Status"),
	);

	my $lbl_status = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Logged out"),
	);
	$lbl_status->SetFont(
		Wx::Font->new( Wx::wxNORMAL_FONT->GetPointSize, 70, 90, 92, 0, "" )
	);

	my $line1 = Wx::StaticLine->new(
		$self,
		-1,
		Wx::wxDefaultPosition,
		Wx::wxDefaultSize,
		Wx::wxLI_HORIZONTAL,
	);

	my $m_staticText2 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Username"),
	);

	my $txt_login = Wx::TextCtrl->new(
		$self,
		-1,
		"",
		Wx::wxDefaultPosition,
		Wx::wxDefaultSize,
	);

	my $m_staticText3 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Password"),
	);

	my $txt_password = Wx::TextCtrl->new(
		$self,
		-1,
		"",
		Wx::wxDefaultPosition,
		Wx::wxDefaultSize,
		Wx::wxTE_PASSWORD,
	);

	my $btn_login = Wx::Button->new(
		$self,
		-1,
		Wx::gettext("Login"),
	);

	Wx::Event::EVT_BUTTON(
		$self,
		$btn_login,
		sub {
			shift->btn_login(@_);
		},
	);

	my $m_staticText5 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Username"),
	);

	my $txt_username = Wx::TextCtrl->new(
		$self,
		-1,
		"",
		Wx::wxDefaultPosition,
		Wx::wxDefaultSize,
	);

	my $m_staticText6 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Password"),
	);

	my $txt_pw = Wx::TextCtrl->new(
		$self,
		-1,
		"",
		Wx::wxDefaultPosition,
		Wx::wxDefaultSize,
	);

	my $m_staticText7 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Confirm"),
	);

	my $txt_pw_confirm = Wx::TextCtrl->new(
		$self,
		-1,
		"",
		Wx::wxDefaultPosition,
		Wx::wxDefaultSize,
	);

	my $m_staticText8 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Email"),
	);

	my $txt_email = Wx::TextCtrl->new(
		$self,
		-1,
		"",
		Wx::wxDefaultPosition,
		Wx::wxDefaultSize,
	);

	my $m_staticText9 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Confirm"),
	);

	my $txt_email_confirm = Wx::TextCtrl->new(
		$self,
		-1,
		"",
		Wx::wxDefaultPosition,
		Wx::wxDefaultSize,
	);

	my $btn_register = Wx::Button->new(
		$self,
		-1,
		Wx::gettext("Register"),
	);

	Wx::Event::EVT_BUTTON(
		$self,
		$btn_register,
		sub {
			shift->btn_register(@_);
		},
	);

	my $line = Wx::StaticLine->new(
		$self,
		-1,
		Wx::wxDefaultPosition,
		Wx::wxDefaultSize,
		Wx::wxLI_HORIZONTAL,
	);

	my $btn_local = Wx::Button->new(
		$self,
		-1,
		Wx::gettext("Upload"),
	);
	$btn_local->Disable;

	Wx::Event::EVT_BUTTON(
		$self,
		$btn_local,
		sub {
			shift->btn_local(@_);
		},
	);

	my $btn_remote = Wx::Button->new(
		$self,
		-1,
		Wx::gettext("Download"),
	);
	$btn_remote->Disable;

	Wx::Event::EVT_BUTTON(
		$self,
		$btn_remote,
		sub {
			shift->btn_remote(@_);
		},
	);

	my $btn_delete = Wx::Button->new(
		$self,
		-1,
		Wx::gettext("Delete"),
	);
	$btn_delete->Disable;

	Wx::Event::EVT_BUTTON(
		$self,
		$btn_delete,
		sub {
			shift->btn_delete(@_);
		},
	);

	my $btn_ok = Wx::Button->new(
		$self,
		Wx::wxID_OK,
		Wx::gettext("Close"),
	);

	Wx::Event::EVT_BUTTON(
		$self,
		$btn_ok,
		sub {
			shift->btn_ok(@_);
		},
	);

	my $fgSizer3 = Wx::FlexGridSizer->new( 2, 2, 0, 0 );
	$fgSizer3->AddGrowableCol(1);
	$fgSizer3->SetFlexibleDirection(Wx::wxBOTH);
	$fgSizer3->SetNonFlexibleGrowMode(Wx::wxFLEX_GROWMODE_SPECIFIED);
	$fgSizer3->Add( $m_staticText12, 0, Wx::wxALIGN_CENTER_VERTICAL | Wx::wxALL, 3 );
	$fgSizer3->Add( $txt_remote, 0, Wx::wxALL | Wx::wxEXPAND, 3 );
	$fgSizer3->Add( $m_staticText13, 0, Wx::wxALL, 3 );
	$fgSizer3->Add( $lbl_status, 1, Wx::wxALIGN_CENTER_VERTICAL | Wx::wxALL | Wx::wxEXPAND, 3 );

	my $fgSizer1 = Wx::FlexGridSizer->new( 3, 2, 0, 0 );
	$fgSizer1->AddGrowableCol(1);
	$fgSizer1->SetFlexibleDirection(Wx::wxHORIZONTAL);
	$fgSizer1->SetNonFlexibleGrowMode(Wx::wxFLEX_GROWMODE_SPECIFIED);
	$fgSizer1->Add( $m_staticText2, 0, Wx::wxALIGN_CENTER_VERTICAL | Wx::wxALL, 3 );
	$fgSizer1->Add( $txt_login, 1, Wx::wxALL | Wx::wxEXPAND, 3 );
	$fgSizer1->Add( $m_staticText3, 0, Wx::wxALIGN_CENTER_VERTICAL | Wx::wxALL, 3 );
	$fgSizer1->Add( $txt_password, 0, Wx::wxALL | Wx::wxEXPAND, 3 );
	$fgSizer1->Add( 0, 0, 1, Wx::wxEXPAND, 5 );
	$fgSizer1->Add( $btn_login, 0, Wx::wxALIGN_RIGHT | Wx::wxALL, 3 );

	my $sbSizer1 = Wx::StaticBoxSizer->new(
		Wx::StaticBox->new(
			$self,
			-1,
			Wx::gettext("Authentication"),
		),
		Wx::wxVERTICAL,
	);
	$sbSizer1->Add( $fgSizer1, 0, Wx::wxEXPAND, 5 );

	my $fgSizer2 = Wx::FlexGridSizer->new( 6, 2, 0, 0 );
	$fgSizer2->AddGrowableCol(1);
	$fgSizer2->SetFlexibleDirection(Wx::wxHORIZONTAL);
	$fgSizer2->SetNonFlexibleGrowMode(Wx::wxFLEX_GROWMODE_SPECIFIED);
	$fgSizer2->Add( $m_staticText5, 0, Wx::wxALIGN_CENTER_VERTICAL | Wx::wxALL, 3 );
	$fgSizer2->Add( $txt_username, 0, Wx::wxALL | Wx::wxEXPAND, 3 );
	$fgSizer2->Add( $m_staticText6, 0, Wx::wxALIGN_CENTER_VERTICAL | Wx::wxALL, 3 );
	$fgSizer2->Add( $txt_pw, 0, Wx::wxALL | Wx::wxEXPAND, 3 );
	$fgSizer2->Add( $m_staticText7, 0, Wx::wxALIGN_CENTER_VERTICAL | Wx::wxALL, 3 );
	$fgSizer2->Add( $txt_pw_confirm, 0, Wx::wxALL | Wx::wxEXPAND, 3 );
	$fgSizer2->Add( $m_staticText8, 0, Wx::wxALIGN_CENTER_VERTICAL | Wx::wxALL, 3 );
	$fgSizer2->Add( $txt_email, 0, Wx::wxALL | Wx::wxEXPAND, 3 );
	$fgSizer2->Add( $m_staticText9, 0, Wx::wxALIGN_CENTER_VERTICAL | Wx::wxALL, 3 );
	$fgSizer2->Add( $txt_email_confirm, 0, Wx::wxALL | Wx::wxEXPAND, 3 );
	$fgSizer2->Add( 0, 0, 1, Wx::wxEXPAND, 5 );
	$fgSizer2->Add( $btn_register, 0, Wx::wxALIGN_RIGHT | Wx::wxALL, 3 );

	my $sbSizer2 = Wx::StaticBoxSizer->new(
		Wx::StaticBox->new(
			$self,
			-1,
			Wx::gettext("Registration"),
		),
		Wx::wxVERTICAL,
	);
	$sbSizer2->Add( $fgSizer2, 1, Wx::wxEXPAND, 5 );

	my $bSizer7 = Wx::BoxSizer->new(Wx::wxHORIZONTAL);
	$bSizer7->Add( $sbSizer1, 1, Wx::wxEXPAND, 5 );
	$bSizer7->Add( 10, 0, 0, Wx::wxEXPAND, 5 );
	$bSizer7->Add( $sbSizer2, 1, Wx::wxEXPAND, 5 );

	my $buttons = Wx::BoxSizer->new(Wx::wxHORIZONTAL);
	$buttons->Add( $btn_local, 0, Wx::wxALL, 3 );
	$buttons->Add( $btn_remote, 0, Wx::wxALL, 3 );
	$buttons->Add( $btn_delete, 0, Wx::wxALL, 3 );
	$buttons->Add( 50, 0, 1, Wx::wxEXPAND, 3 );
	$buttons->Add( $btn_ok, 0, Wx::wxALL, 3 );

	my $vsizer = Wx::BoxSizer->new(Wx::wxVERTICAL);
	$vsizer->Add( $fgSizer3, 0, Wx::wxEXPAND, 5 );
	$vsizer->Add( $line1, 0, Wx::wxBOTTOM | Wx::wxEXPAND | Wx::wxTOP, 5 );
	$vsizer->Add( $bSizer7, 1, Wx::wxEXPAND, 5 );
	$vsizer->Add( $line, 0, Wx::wxBOTTOM | Wx::wxEXPAND | Wx::wxTOP, 5 );
	$vsizer->Add( $buttons, 0, Wx::wxALIGN_RIGHT | Wx::wxEXPAND, 0 );

	my $hsizer = Wx::BoxSizer->new(Wx::wxHORIZONTAL);
	$hsizer->Add( $vsizer, 1, Wx::wxALL | Wx::wxEXPAND, 5 );

	$self->SetSizer($hsizer);
	$self->Layout;
	$hsizer->Fit($self);

	return $self;
}

sub btn_login {
	$_[0]->main->error('Handler method btn_login for event btn_login.OnButtonClick not implemented');
}

sub btn_register {
	$_[0]->main->error('Handler method btn_register for event btn_register.OnButtonClick not implemented');
}

sub btn_local {
	$_[0]->main->error('Handler method btn_local for event btn_local.OnButtonClick not implemented');
}

sub btn_remote {
	$_[0]->main->error('Handler method btn_remote for event btn_remote.OnButtonClick not implemented');
}

sub btn_delete {
	$_[0]->main->error('Handler method btn_delete for event btn_delete.OnButtonClick not implemented');
}

sub btn_ok {
	$_[0]->main->error('Handler method btn_ok for event btn_ok.OnButtonClick not implemented');
}

1;

# Copyright 2008-2011 The Padre development team as listed in Padre.pm.
# LICENSE
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl 5 itself.

